/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Dict
 *
 * The Initial Developer of the Original Code is David Costanzo
 * Portions created by the Initial Developer are Copyright (C) 2003-2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/////////////////////////////////////////////////////////////////////////////
// HACK: this block of code was pref.js. 
// For some reason we can't include it from context.xul
// 
const PREFNAME_DEFAULTSERVERNAME = "extensions.dict.defaultservername";
const PREFNAME_DEFAULTSERVERPORT = "extensions.dict.defaultserverport";

// Get the default server name, or use a default value
// Does not throw an exception.
function dictGetDefaultServerName() {

    var pref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);

    try {
         return pref.getCharPref(PREFNAME_DEFAULTSERVERNAME);
    }
    catch (e) {
    }

    // something is wrong with the preferences,
    // return a default value
    return 'dict.org';
}

// Get the default server port, or use a default value
// Does not throw an exception.
function dictGetDefaultServerPort() {

    var pref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);

    try {
         return pref.getCharPref(PREFNAME_DEFAULTSERVERPORT);
    }
    catch (e) {
    }

    try {
        // maybe it's stored as an int
        return pref.getIntPref(PREFNAME_DEFAULTSERVERPORT);
    }
    catch (e) {
    }

    // something is wrong with the preferences,
    // return a default value
    return '2628';
}
// END HACK
/////////////////////////////////////////////////////////////////////////////


// dictInit will be called every time a new browser window is created
window.addEventListener("load", dictDefinitionInit, false);

// DictListener class, which is passed in as the "listener" parameter
// to DictClient::defineWord().
function DictListener () {
}

// wraps some HTML inside HTML/BODY tag and adds a title
DictListener.prototype = 
{

    createHtmlDocument : function (title, body)
    {
        var htmlDocument;
    
        htmlDocument = 
            '<HTML>\n'    +
            '  <HEAD>\n'  + 
            '      <TITLE>' + dictUtf8ToCharacterEntities(title) + '</TITLE>\n' +
            '  </HEAD>\n' +
            '  <BODY>\n'  +
            dictUtf8ToCharacterEntities(body) +
            '  </BODY>\n' +
            '</HTML>\n';
        
        return htmlDocument;
    },

    // Creates a new window and sets the document to be the given HTML.
    // Resizes the window to fit the content.
    createWindow : function ()
    {
        // Determine if were are in "single user mode", as implemented
        // by the TabExtensionBrowser.  If we are, then the new window
        // will appear in a new tab.  In this case, we do NOT want to
        // resize the window, because that would resize the window
        // for all tabs, which is very annoying.
        var prefClass = Components.classes["@mozilla.org/preferences-service;1"];
        var pref      = prefClass.getService(Components.interfaces.nsIPrefBranch);
        
        var mustResizeWindow = true;
        try {
            var windowHookMode = pref.getIntPref('browser.tabs.extensions.window_hook_mode');
            if (windowHookMode == 2) {
                // we are in single window mode.
                mustResizeWindow = false;
            }
        }
        catch (e) {
        }
        

        if (mustResizeWindow) {

            window.sizeToContent();
            window.innerHeight += 70;

            if (window.outerWidth < window.innerWidth) {
                // The window became narrower than the content. 
                // This should never happen, but it may be happening 
                // because of bug #7097--a bug in DownloadStatusBar
                // when it has over 40 items.

                // I can't fix DownloadStatusBar, but I can be defensive
                // and resize the window back to its original width.
                window.outerWidth = window.innerWidth;
            }

            // If the page is too long to display without scrollbars, then shorten it
            if (600 < window.innerHeight) {
                // limit the height
                window.innerHeight = 600;
            
                // make wider to account for the width of the 
                // vertical scrollbar.  If we don't do this, a 
                // horizontal scrollbar will appear.
                window.innerWidth += 20;
            }
        }
    },

    //
    // The next three functions are the DictListener interface
    //

    // Called when the DictClient has definitions
    onHaveDefinitions : function (word, definitions)
    {
        alert('onHaveDefinitions called');
        var vbox = window.document.getElementById('content');

        // count the number of sources -- used for formatting
        var totalSources = 0;
        for (source in definitions) {
            totalSources++;
        }

        var sourceIndex = 0;
        for (source in definitions) {

            var bold = window.document.createElement('html:b');
            vbox.appendChild(bold);

            var dbDiv = window.document.createElement('html:div');
            dbDiv.setAttribute('class', 'dict-database');
            bold.appendChild(dbDiv);

            var sourceNode = window.document.createTextNode(source);
            bold.appendChild(sourceNode);



            var codeNode = window.document.createElement('html:code');
            vbox.appendChild(codeNode);

            var codeSpan = window.document.createElement('html:span');
            codeSpan.setAttribute('class', 'dict-definition');
            codeNode.appendChild(codeSpan);

            var i;
            for (i = 0; i < definitions[source].length; i++) {

                var deftext = definitions[source][i].text;

                // normalize all definitions so that they end in a single newline
                deftext = deftext.replace(/\s*$/, '\n');
                                
                // Escape HTML entities
                deftext = deftext.replace(/&/g, '&amp;');
                deftext = deftext.replace(/</g, '&lt;');
                            
                // replace newlines with <BR> and whitespace with
                // non-breaking spaces to preserve formatting.
                // This is better than using <PRE> because it
                // allows us to use other mark-up tags.
                deftext = deftext.replace(/\n/g, '<BR>');
                deftext = deftext.replace(/ /g, '&nbsp;');
                                
                // Words that are cross-referenced by WordNet show up 
                // in braces. e.g., "{word}" and "{word(P)}"
                deftext = deftext.replace(/{(.*?)(\(.*\))?}/g, '$1');
                
                // add a newline after each <BR> to make the 
                // HTML source more readable.
                deftext = deftext.replace(/<BR>/g, '<BR>\n');
                
                // append an extra newline after everything but the
                // the final definition from the final source.
                // (Otherwise there'd be a strange-looking blank space
                // at the bottom of the page.)
                if (sourceIndex + 1 != totalSources ||
                    i + 1           != definitions[source].length) {
                    deftext += '<BR>\n';
                }
                
                var defNode = window.document.createTextNode(deftext);
                codeSpan.appendChild(defNode);
            }
            
            sourceIndex++;
        }

        this.createWindow();
        return;

        // Format the definitions into HTML
        var htmlBody = '<DIV class="dict-page" style="width:475">\n';

        // count the number of sources -- used for formatting
        var totalSources = 0;
        for (source in definitions) {
            totalSources++;
        }

        var sourceIndex = 0;
        for (source in definitions) {

            htmlBody += '<B><DIV class="dict-database">';
            htmlBody += source;
            htmlBody += '</DIV></B>\n';
            
            htmlBody += '<CODE><SPAN class="dict-definition">\n';

            var i;
            for (i = 0; i < definitions[source].length; i++) {

                var deftext = definitions[source][i].text;

                // normalize all definitions so that they end in a single newline
                deftext = deftext.replace(/\s*$/, '\n');
                                
                // Escape HTML entities
                deftext = deftext.replace(/&/g, '&amp;');
                deftext = deftext.replace(/</g, '&lt;');
                            
                // replace newlines with <BR> and whitespace with
                // non-breaking spaces to preserve formatting.
                // This is better than using <PRE> because it
                // allows us to use other mark-up tags.
                deftext = deftext.replace(/\n/g, '<BR>');
                deftext = deftext.replace(/ /g, '&nbsp;');
                                
                // Words that are cross-referenced by WordNet show up 
                // in braces. e.g., "{word}" and "{word(P)}"
                deftext = deftext.replace(/{(.*?)(\(.*\))?}/g, '$1');
                
                // add a newline after each <BR> to make the 
                // HTML source more readable.
                deftext = deftext.replace(/<BR>/g, '<BR>\n');
                
                // append an extra newline after everything but the
                // the final definition from the final source.
                // (Otherwise there'd be a strange-looking blank space
                // at the bottom of the page.)
                if (sourceIndex + 1 != totalSources ||
                    i + 1           != definitions[source].length) {
                    deftext += '<BR>\n';
                }
                
                htmlBody += deftext;
            }
            
            htmlBody += '</SPAN></CODE>\n';
            sourceIndex++;
        }
        
        htmlBody += '</DIV>\n';
        
        var htmlDocument = this.createHtmlDocument(word, htmlBody);
        
        try {
            this.createWindow();
        }
        catch (e) {
            alert(e);
        }
    },

    // Called when the DictClient has closest matches
    onHaveClosestMatches : function (word, matches)
    {
        alert('onHaveClosestMatches called');

        var vbox = window.document.getElementById('content');

        //'<DIV class="dict-page" style="width:450">\n' + 
                        //'<B><DIV class="dict-heading">Closest Matches</DIV></B><BR>\n' + 
                        //'<DIV class="dict-text">\n' + 

        var explainationNode = window.document.createElement('label');
        explainationNode.setAttribute(
            'value', 
            'No definitions for "' + word + '" were found, ' +
                'but definitions for similar-looking words were found.  ' +
                'To define one of these close matches, highlight ' +
                'the word below, right-click, and select "Define" ' +
                'from the pop-up menu.');
        vbox.appendChild(explainationNode);

        for (match in matches) {
            var quotelessMatch = match.replace(/"/g /*"*/, '');
            var link = window.document.createElement('button');
            link.setAttribute('label', quotelessMatch);
            link.setAttribute('oncommand', 'dictDefineWord(\'' + quotelessMatch + '\');');
            
            vbox.appendChild(link);
        }

        this.createWindow();
        alert('onHaveClosestMatches returning');
        return;

        var htmlBody;

        htmlBody = 
            '<script language="javascript" type="application/x-javascript" src="chrome://dict/content/connection-xpcom.js"></script>\n' +
            '<script language="javascript" type="application/x-javascript" src="chrome://dict/content/dictclient.js"></script>\n' +
            '<script language="javascript" type="application/x-javascript" src="chrome://dict/content/utf8.js"></script>\n' +
        '<script language="javascript" type="application/x-javascript" src="chrome://dict/content/definition.js"></script>\n';

        htmlBody +=
            '<DIV class="dict-page" style="width:450">\n' + 
                '<B><DIV class="dict-heading">Closest Matches</DIV></B><BR>\n' + 
                '<DIV class="dict-text">\n' + 
                    'No definitions for "' + word + '" were found, ' +
                    'but definitions for similar-looking words were found.  ' +
                    'To define one of these close matches, highlight ' +
                    'the word below, right-click, and select "Define" ' +
                    'from the pop-up menu.' +
                 '</DIV>\n';

        // add a spacer between the text and list of words
        htmlBody += '<HR class="dict-horizontal-rule">\n';

        for (match in matches) {
            var quotelessMatch = match.replace(/"/g, ''); //" balance quotes for syntax highlighting
            htmlBody += '<A href="javascript:dictDefineWord(\'' + quotelessMatch + '\');">' + quotelessMatch + '<A><BR>\n';
        }

        htmlBody += '</DIV>\n';


        var htmlDocument = this.createHtmlDocument(
            word + ' - Closest Matches',
            htmlBody);
        
        this.createWindow();
    },

    onNoDefinitionsFound : function (word) 
    {
        alert('No definitions were found for "' + dictUtf8ToUcs4(word) + '".\n');
    },
    
    onError : function (error)
    {
        alert(dictUtf8ToUcs4(error));
    }
}


function dictDefineWord(word)
{
    window.open(
        'chrome://dict/content/definition.xul?' + word,
        '',
        '');
}

// Lookup the word when the document is loaded
function dictDefinitionInit() {

    // the word to define is embedded in the URL.
    var url = window.location.href;
    var queryStartIndex = url.indexOf('?');
    var word = url.slice(queryStartIndex + 1);

    alert('defining: ' + word);

    try {
        // read the host and port from the preferences
        var host = dictGetDefaultServerName();
        var port = dictGetDefaultServerPort();

        var listener = new DictListener();
        var client   = new DictClient();
        client.defineWord(host, port, word, listener);
    }
    catch (ex) {
        alert(ex);
    }
}
